//
//	Includes
//
#include	"MCF52233.h"
#include	"ColdFire.h"
#include	"ProtoType.h"

//
//	Prototypes
//
UINT32 Init(UINT32 arg);
UINT32 Size(UINT32 arg);
UINT32 Voice(UINT32 arg);
UINT32 Clean(UINT32 arg);
UINT32 GetQueue(void);
UINT32 PutQueue(UINT32 data);
__declspec(interrupt) TimerHandler(void);

//
//	Constants
//

//
//	Vars
//

UINT16	*BufAdrs;		// Voice data buffer adrs
UINT32	BufSize;		// Voice data buffer size
UINT32	RingIn;			// Queue in pointer
UINT32	RingOut;		// Queue out pointer
UINT32	DataSize;		// Source size
UINT32	LastVal;		// Last value of queue

//
//	Init user driver
//
UINT32 Init(UINT32 arg)	// arg is buffer size
{
//	Setup buffer
	BufSize = arg / 2;
	BufAdrs = MemoryAlloc((UINT16)arg);	// Alloc voice data buffer
	if(BufAdrs == NULL)	// Success?
		return 0;				// Error return
	RingIn = 0;				// Clear queue in pointer
	RingOut = 0;			// Clear queue out pointer
//	Setup PWM
	MCF_GPIO_PNQPAR = 0x0c;	// Port NQ Bit1 is PWM1 pin

	MCF_PWM_PWMCTL = 0x10;	// Set PWM1 and PWM0 combine mode (16bit)
	MCF_PWM_PWMPER0 = 0x03;	// Set PWM length 0x400 (29.3KHz PWM) 
	MCF_PWM_PWMPER1 = 0xff;	// Set PWM length 0x400 (29.3KHz PWM)

	MCF_PWM_PWMDTY0 = 0x02;	// Set initial PWM value (center)
	MCF_PWM_PWMDTY1 = 0x00;	// Set initial PWM value (center)

	MCF_PWM_PWMPOL = 0x02;	// Set output priority reverse
	MCF_PWM_PWME = 0x02;		// Enable PWM module
//	Setup data feed timer
	*(UINT32*)0x2000014c = (UINT32)TimerHandler;	// Set vector (interrupt 19)

	MCF_INTC0_ICR19 = MCF_INTC_ICR_IL(5) | MCF_INTC_ICR_IP(5);// Set interrupt priority
	MCF_INTC0_IMRL &= ~MCF_INTC_IMRL_INT_MASK19;	// Clear interrupt 19 mask bit

	MCF_DTIM0_DTMR = 0x1a;	// Use internal clock & repeat & enable interrupt
	MCF_DTIM0_DTRR = 7500 - 1;	// Set timer interval 125ns (8KHz)
	MCF_DTIM0_DTMR |= 1;		// Enable timer module

	return arg;		// Return with word size (normal return)
}

//
//	Set data length
//
UINT32 Size(UINT32 arg)
{
	DataSize = arg;				// Save data size for later
	return 0;
}

//
//	Send WAVE data to PWM module
//
UINT32 Voice(UINT32 arg)
{
INT32	size;							// Transfer size
UINT16 *ptr;						// Transfer pointer
	size = (INT32)DataSize;			// Make copy of data size
	ptr = (UINT16*)arg;		// Make copy of data buffer
	while(size > 0)				// Loop for data size
	{
		if(PutQueue(*ptr++))// Set WAVE data to queue
			return 1;					// No space then error return
		size -= 2;					// Count down data size
	}
	return arg;						// Return with address for SilentC memory release
}

//
//	Clean up
//
UINT32 Clean(UINT32 arg)
{
	MCF_PWM_PWME = 0;			// Disable PWM
	MCF_DTIM0_DTMR = 0;		// Disable feed timer
	MemoryFree(BufAdrs);	// Release buffer
	return arg;							// Normal return
}

//
//	Get data from queue
//
UINT32 GetQueue(void)
{
	if(RingIn == RingOut)	// No data in the queue?
		return LastVal;			// Return last value
	LastVal = BufAdrs[RingOut++];	// Get data from data queue
	if(RingOut ==	BufSize)// Reach end of queue?
		RingOut = 0;				// Set get index to top of buffer
	return LastVal;							// Return normal voice data
}

//
//	Put data to queue
//
UINT32 PutQueue(UINT32 data)
{
UINT32 ptr;							// Queue pointer work
	ptr = RingIn + 1;			// Get advanced put pointer
	if(ptr == BufSize)		// Reach end of queue?
		ptr = 0;						// Set put index to top of buffer
	if(ptr == RingOut)		// Queue is full?
		return 1;						// Full return
	BufAdrs[RingIn] = (UINT16)data;	// Put data to queue
	RingIn = ptr;					// Set new put pointer
	return 0;							// Normal return
}

//
//	Sci0 interrupt handler
//
__declspec(interrupt) TimerHandler(void)
{
UINT16 v;
	MCF_DTIM0_DTER = 2;		// Clear timer interrupt flag
	v = (UINT16)GetQueue();				// Get WAVE data from queue
	v = (UINT16)(((v >> 14) & 3 | (v << 2) & 0x3fc) + 0x200);	// Swap high and low
	MCF_PWM_PWMDTY0 = (UINT8)((v >> 8) & 3);	// Set high WAVE data to PWM
	MCF_PWM_PWMDTY1 = (UINT8)(v & 255);// Set low WAVE data to PWM
}
